/**
 * trayicon.cpp
 *
 * part of eeetray
 * written by SvOlli
 * distributed under the terms of the GPL
 */

#include <QtGui>

#include "trayicon.hpp"


TrayIcon::TrayIcon( QWidget *parent )
: QSystemTrayIcon( parent )
{
}


bool TrayIcon::event( QEvent *e )
{
   if( e->type() == QEvent::ToolTip )
   {
      bool discharging = false;
      int present = 0;
      int remain  = 0;
      int hours   = 0;
      int minutes = 0;
      QRegExp getnum( "(\\d+)" );
      
      QFile file("/proc/acpi/battery/BAT0/state");
      file.open( QIODevice::ReadOnly | QIODevice::Text );
      QTextStream stream( &file );
      stream.setCodec( "UTF-8" );
      
      QString line;
      
      do
      {
         line = stream.readLine();
         if( line.startsWith( "charging state:" ) )
         {
            discharging = line.endsWith( "discharging" );
         }
         if( line.startsWith( "present rate:" ) )
         {
            if( getnum.indexIn( line ) >= 0 )
            {
               present = getnum.cap().toInt();
            }
         }
         if( line.startsWith( "remaining capacity:" ) )
         {
            if( getnum.indexIn( line ) >= 0 )
            {
               remain = getnum.cap().toInt();
            }
         }
      }
      while( !stream.atEnd() );
      
      file.close();
      if( discharging && remain && present )
      {
         hours = remain / present;
         minutes = (60 * remain / present) % 60;
         line.clear();
         QTextStream( &line ) << hours << ":" << qSetFieldWidth(2) << qSetPadChar('0') << minutes;
         QHelpEvent *he = static_cast<QHelpEvent *>(e);
         QToolTip::showText( he->globalPos(), line );
      }
      else
      {
         QToolTip::hideText();
      }
   }
   return QSystemTrayIcon::event( e );
}
