/**
 * window.cpp
 *
 * part of eeetray
 * written by SvOlli
 * distributed under the terms of the GPL
 */

#include <QtGui>

#include "window.hpp"
#include "trayicon.hpp"
extern "C" {
#include "backend.h"
};


Window::Window()
: QDialog()
, mpTrayIcon( new /*QSystem*/TrayIcon(this) )
, mpMainMenu( new QMenu(this) )
, mpShutdownMenu( new QMenu(this) )
/* shortcuts used: 3ABCDELOQRSW */
, mpActionReboot( new QAction(tr("&Reboot"), this) )
, mpActionPoweroff( new QAction(tr("Power&off"), this) )
, mpActionWirelessOff( new QAction(tr("A&ll Wireless"), this) )
, mpActionQuit( new QAction(tr("&Quit"), this) )
, mpActionAbout( new QAction(tr("&About"), this) )
, mpActionCardreader( new QAction(tr("&Cardreader"), this) )
, mpActionWebcam( new QAction(tr("W&ebcam"), this) )
, mpActionBluetooth( new QAction(tr("&Bluetooth"), this) )
, mpActionThreeG( new QAction(tr("&3G Modem"), this) )
, mpActionWLAN( new QAction(tr("&WLAN"), this) )
, mpActionSSH( new QAction(tr("&SSH"), this) )
, mMenuLocked( false )
, mExecThread( )
, mNormalIcon( "/usr/share/pixmaps/ktip-16.xpm" )
, mActiveIcon( "/usr/share/pixmaps/keditbookmarks-16.xpm" )
{
   init_backend();
   
   mpActionCardreader->setCheckable( true );
   mpActionWebcam->setCheckable( true );
   mpActionBluetooth->setCheckable( true );
   mpActionThreeG->setCheckable( true );
   mpActionWLAN->setCheckable( true );
   mpActionSSH->setCheckable( true );
   
   mpShutdownMenu->setTitle(tr("Shut&down"));
   mpShutdownMenu->addAction(mpActionQuit);
   mpShutdownMenu->addAction(mpActionWirelessOff);
   mpShutdownMenu->addAction(mpActionReboot);
   mpShutdownMenu->addAction(mpActionPoweroff);

   mpMainMenu->addMenu(mpShutdownMenu);
   mpMainMenu->addAction(mpActionAbout);
   mpMainMenu->addSeparator();
   if( ::test_cardreader() >= 0 )
   {
      mpMainMenu->addAction(mpActionCardreader);
   }
   if( ::test_webcam() >= 0 )
   {
      mpMainMenu->addAction(mpActionWebcam);
   }
   if( ::test_bluetooth() >= 0 )
   {
      mpMainMenu->addAction(mpActionBluetooth);
   }
   if( ::test_threeg() >= 0 )
   {
      mpMainMenu->addAction(mpActionThreeG);
   }
   if( ::test_wlan() >= 0 )
   {
      mpMainMenu->addAction(mpActionWLAN);
   }
   if( ::test_ssh() >= 0 )
   {
      mpMainMenu->addAction(mpActionSSH);
   }
   mpTrayIcon->setIcon( mNormalIcon );

   connect( mpMainMenu, SIGNAL(triggered(QAction*)), 
            this, SLOT(handleMenu(QAction*)) );
   connect( mpShutdownMenu, SIGNAL(triggered(QAction*)), 
            this, SLOT(handleMenu(QAction*)) );
   connect( mpTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)) );
   connect( &mExecThread, SIGNAL(started()),
            this, SLOT(execStarted()) );
   connect( &mExecThread, SIGNAL(finished()),
            this, SLOT(execFinished()) );
   connect( &mExecThread, SIGNAL(terminated()),
            this, SLOT(execFinished()) );
   
   QLabel *about = new QLabel( tr("eeetray v0.8 written by SvOlli<br>"
                  "<a href='http://svolli.org/software/eeepc/#eeetray'>"
                  "svolli.org/software/eeepc/</a>") );
   about->setOpenExternalLinks( true );
   QVBoxLayout *mainLayout = new QVBoxLayout;
   mainLayout->addWidget( about );
   setLayout(mainLayout);

   mpTrayIcon->show();

   setWindowTitle(tr("eeetray"));
   resize(100, 50);
}

void Window::closeEvent(QCloseEvent *event)
{
   hide();
   event->ignore();
}

void Window::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
   if( mMenuLocked )
   {
      return;
   }
   
   switch (reason)
   {
      case QSystemTrayIcon::Trigger:
      case QSystemTrayIcon::MiddleClick:
      case QSystemTrayIcon::Context:
         mpActionCardreader->setChecked( ::test_cardreader() > 0 );
         mpActionWebcam->setChecked( ::test_webcam() > 0 );
         mpActionBluetooth->setChecked( ::test_bluetooth() > 0 );
         mpActionThreeG->setChecked( ::test_threeg() > 0 );
         mpActionWLAN->setChecked( ::test_wlan() > 0 );
         mpActionSSH->setChecked( ::test_ssh() > 0 );
      
         mpMainMenu->move( QCursor::pos() );
         mpMainMenu->exec();
         break;
      case QSystemTrayIcon::DoubleClick:
         break;
      default:
         break;
   }
}

void Window::handleMenu( QAction *action )
{
   mExecThread.mCommand = BackendThread::none;
   
   if( action == mpActionAbout )
   {
      show();
      QApplication::aboutQt();
      return;
   }
   else if( action == mpActionQuit )
   {
      QApplication::quit();
      return;
   }
   else if( action == mpActionReboot )
   {
      mExecThread.mCommand = BackendThread::reboot;
   }
   else if( action == mpActionPoweroff )
   {
      mExecThread.mCommand = BackendThread::poweroff;
   }
   else if( action == mpActionWirelessOff )
   {
      mExecThread.mCommand = BackendThread::wirelessoff;
   }
   else if( action == mpActionCardreader )
   {
      mExecThread.mCommand = BackendThread::cardreader;
   }
   else if( action == mpActionWebcam )
   {
      mExecThread.mCommand = BackendThread::webcam;
   }
   else if( action == mpActionBluetooth )
   {
      mExecThread.mCommand = BackendThread::bluetooth;
   }
   else if( action == mpActionThreeG )
   {
      mExecThread.mCommand = BackendThread::threeg;
   }
   else if( action == mpActionWLAN )
   {
      mExecThread.mCommand = BackendThread::wlan;
   }
   else if( action == mpActionSSH )
   {
      mExecThread.mCommand = BackendThread::ssh;
   }
   
   if( mExecThread.mCommand != BackendThread::none )
   {
      mExecThread.start();
   }
}


void Window::execStarted()
{
   mMenuLocked = true;
   mpTrayIcon->setIcon( mActiveIcon );
}

void Window::execFinished()
{
   mpTrayIcon->setIcon( mNormalIcon );
   mMenuLocked = false;
}
